const fs = require("fs");
const { join, dirname } = require("path");
const TerserPlugin = require("terser-webpack-plugin");

function findRootDir(dir) {
    if (dir === "/") {
        return "/";
    }
    if (fs.existsSync(join(dir, "package.json"))) {
        return dir;
    }
    return findRootDir(dirname(dir));
}

const RootDir = findRootDir(process.cwd());

module.exports = {
    production: "production",
    development: "development",
    dir(mode) {
        if (mode === this.development) {
            return "./debug";
        }
        return "./dist";
    },
    getDir(dir) {
        if (!dir) {
            return RootDir;
        }
        return join(RootDir, dir);
    },
    entrys(isProduction) {
        if (!isProduction) {
            return { main: "./src/main.js" };
        }
        const SRC_DIR = this.getDir("src");
        let data = fs.readdirSync(SRC_DIR),
            result = {};
        data.map((item) => {
            let idx = item.indexOf(".");
            if (idx > -1) {
                if (item.indexOf(".js") > -1) {
                    let sourceFile = join(SRC_DIR, item);
                    let entryName = item.substr(0, idx);
                    result[entryName] = sourceFile;
                }
                return;
            }
            let sourceFile = join(SRC_DIR, item, "main.js");
            if (!fs.existsSync(sourceFile)) {
                return;
            }
            result[item] = sourceFile;
        });
        return result;
    },
    components() {
        let result = [];
        const SRC_DIR = this.getDir("src");
        let data = fs.readdirSync(SRC_DIR);
        data.map((item) => {
            item.indexOf(".") === -1 && result.push(item);
        });
        return result;
    },
    mode(mode) {
        if (mode === this.production) {
            return mode;
        }
        return "development";
    },
    optimization(mode) {
        let config = { splitChunks: false };
        if (mode === this.production) {
            config["minimizer"] = [
                new TerserPlugin({
                    parallel: true,
                    extractComments: false,
                    terserOptions: {
                        format: {
                            ascii_only: true,
                            comments: false,
                        },
                        toplevel: true,
                        ie8: false,
                        compress: {
                            defaults: false,
                            arrows: false,
                            global_defs: {
                                DEBUG: false,
                            },
                            drop_console: true,
                            drop_debugger: true,
                            dead_code: true,
                        },
                    },
                }),
            ];
        }
        return config;
    },
};
